@@ -1,6 +1,6 @@
 /* plaintext.c -  process plaintext packets
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+ *               2006 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -83,14 +84,14 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     if( nooutput )
 	;
     else if( opt.outfile ) {
-	fname = m_alloc( strlen( opt.outfile ) + 1);
+	fname = xmalloc( strlen( opt.outfile ) + 1);
 	strcpy(fname, opt.outfile );
     }
     else if( pt->namelen == 8 && !memcmp( pt->name, "_CONSOLE", 8 ) ) {
 	log_info(_("data not saved; use option \"--output\" to save it\n"));
 	nooutput = 1;
     }
-    else if( !opt.use_embedded_filename ) {
+    else if( !opt.flags.use_embedded_filename ) {
 	fname = make_outfile_name( iobuf_get_real_fname(pt->buf) );
 	if( !fname )
 	    fname = ask_outfile_name( pt->name, pt->namelen );
@@ -99,9 +100,8 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    goto leave;
 	}
     }
-    else {
-	fname = make_printable_string( pt->name, pt->namelen, 0 );
-    }
+    else
+      fname=utf8_to_native(pt->name,pt->namelen,0);
 
     if( nooutput )
 	;
@@ -117,11 +117,11 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	while( !overwrite_filep (fname) ) {
             char *tmp = ask_outfile_name (NULL, 0);
             if ( !tmp || !*tmp ) {
-                m_free (tmp);
+                xfree (tmp);
                 rc = G10ERR_CREATE_FILE;
                 goto leave;
             }
-            m_free (fname);
+            xfree (fname);
             fname = tmp;
         }
     }
@@ -176,8 +176,14 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 #endif /* __riscos__ */
 
     if( !pt->is_partial ) {
-        /* we have an actual length (which might be zero). */
-	assert( !clearsig );
+        /* We have an actual length (which might be zero). */
+
+        if (clearsig) {
+            log_error ("clearsig encountered while not expected\n");
+            rc = G10ERR_UNEXPECTED;
+            goto leave;
+        }
+
 	if( convert ) { /* text mode */
 	    for( ; pt->len; pt->len-- ) {
 		if( (c = iobuf_get(pt->buf)) == -1 ) {
@@ -212,7 +218,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    }
 	}
 	else { /* binary mode */
-	    byte *buffer = m_alloc( 32768 );
+	    byte *buffer = xmalloc( 32768 );
 	    while( pt->len ) {
 		int len = pt->len > 32768 ? 32768 : pt->len;
 		len = iobuf_read( pt->buf, buffer, len );
@@ -220,7 +226,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		    log_error("Problem reading source (%u bytes remaining)\n",
 			      (unsigned)pt->len);
 		    rc = G10ERR_READ_FILE;
-		    m_free( buffer );
+		    xfree( buffer );
 		    goto leave;
 		}
 		if( mfx->md )
@@ -232,7 +238,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 			log_error("Error writing to `%s': %s\n",
 				  fname,"exceeded --max-output limit\n");
 			rc = G10ERR_WRITE_FILE;
-			m_free( buffer );
+			xfree( buffer );
 			goto leave;
 		      }
 		    else if( fwrite( buffer, 1, len, fp ) != len )
@@ -240,13 +246,13 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
 			rc = G10ERR_WRITE_FILE;
-			m_free( buffer );
+			xfree( buffer );
 			goto leave;
 		      }
 		  }
 		pt->len -= len;
 	    }
-	    m_free( buffer );
+	    xfree( buffer );
 	}
     }
     else if( !clearsig ) {
@@ -278,7 +284,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    }
 	}
 	else { /* binary mode */
-	    byte *buffer = m_alloc( 32768 );
+	    byte *buffer = xmalloc( 32768 );
 	    int eof;
 	    for( eof=0; !eof; ) {
 		/* Why do we check for len < 32768:
@@ -301,19 +307,19 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 			log_error("Error writing to `%s': %s\n",
 				  fname,"exceeded --max-output limit\n");
 			rc = G10ERR_WRITE_FILE;
-			m_free( buffer );
+			xfree( buffer );
 			goto leave;
 		      }
 		    else if( fwrite( buffer, 1, len, fp ) != len ) {
 		      log_error("Error writing to `%s': %s\n",
 				fname, strerror(errno) );
 		      rc = G10ERR_WRITE_FILE;
-		      m_free( buffer );
+		      xfree( buffer );
 		      goto leave;
 		    }
 		  }
 	    }
-	    m_free( buffer );
+	    xfree( buffer );
 	}
 	pt->buf = NULL;
     }
@@ -381,7 +387,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
   leave:
     if( fp && fp != stdout )
 	fclose(fp);
-    m_free(fname);
+    xfree(fname);
     return rc;
 }
 
@@ -445,10 +451,16 @@ ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
 	int any=0;
 	tty_printf(_("Detached signature.\n"));
 	do {
-	    m_free(answer);
-	    answer = cpr_get("detached_signature.filename",
+	    char *name;
+	    xfree(answer);
+	    tty_enable_completion(NULL);
+	    name = cpr_get("detached_signature.filename",
 			   _("Please enter name of data file: "));
+	    tty_disable_completion();
 	    cpr_kill_prompt();
+	    answer=make_filename(name,(void *)NULL);
+	    xfree(name);
+
 	    if( any && !*answer ) {
 		rc = G10ERR_READ_FILE;
 		goto leave;
@@ -483,7 +495,7 @@ ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
     iobuf_close(fp);
 
   leave:
-    m_free(answer);
+    xfree(answer);
     return rc;
 }
 
@@ -534,3 +546,44 @@ hash_datafiles( MD_HANDLE md, MD_HANDLE md2, STRLIST files,
 
     return 0;
 }
+
+
+/* Set up a plaintext packet with the appropriate filename.  If there
+   is a --set-filename, use it (it's already UTF8).  If there is a
+   regular filename, UTF8-ize it if necessary.  If there is no
+   filenames at all, set the field empty. */
+
+PKT_plaintext *
+setup_plaintext_name(const char *filename,IOBUF iobuf)
+{
+  PKT_plaintext *pt;
+
+  if(filename || opt.set_filename)
+    {
+      char *s;
+
+      if(opt.set_filename)
+	s=make_basename(opt.set_filename,iobuf_get_real_fname(iobuf));
+      else if(filename && !opt.flags.utf8_filename)
+	{
+	  char *tmp=native_to_utf8(filename);
+	  s=make_basename(tmp,iobuf_get_real_fname(iobuf));
+	  xfree(tmp);
+	}
+      else
+	s=make_basename(filename,iobuf_get_real_fname(iobuf));
+
+      pt = xmalloc (sizeof *pt + strlen(s) - 1);
+      pt->namelen = strlen (s);
+      memcpy (pt->name, s, pt->namelen);
+      xfree (s);
+    }
+  else
+    {
+      /* no filename */
+      pt = xmalloc (sizeof *pt - 1);
+      pt->namelen = 0;
+    }
+
+  return pt;
+}
